local hcTk = HCToolkit

local UnitIsGroupLeader = UnitIsGroupLeader

function hcTk.ArgusEncounterStart()
    hcTk.Encounter["SoulburstTargets"] = {}
    hcTk.Encounter["SoulbombTarget"] = nil
    hcTk.Encounter["SoulExpire"] = nil
end

function hcTk.ArgusEncounterEnd()
    hcTk.Encounter["SoulburstTargets"] = nil
    hcTk.Encounter["SoulbombTarget"] = nil
    hcTk.Encounter["SoulExpire"] = nil
end

function hcTk.ArgusCombatLog(...)
    local msg = select(2, ...)
    if (hcTk.ArgusCombatLogFns[msg] ~= nil) then
        hcTk.ArgusCombatLogFns[msg](...)
    end
end

function hcTk.ArgusSpellAuraApplied(...)
    local destGUID, _, _, _, spellId = select(8, ...)
    if (spellId == 250669) then -- Soulburst
        hcTk.ArgusSoulburstApplied(destGUID)
    elseif (spellId == 251570) then -- Soulbomb
        hcTk.ArgusSoulbombApplied(destGUID)
    end
end

function hcTk.ArgusSpellAuraRemoved(...)
    local destGUID, _, _, _, spellId = select(8, ...)
    if (hcTk.Group[destGUID] ~= nil and (spellId == 250669 or spellId == 251570)) then
        hcTk.ArgusRemoveSouls(destGUID)
    end
end

function hcTk.ArgusUnitDied(...)
    local destGUID = select(8, ...)
    if (hcTk.Group[destGUID] ~= nil) then
        hcTk.ArgusRemoveSouls(destGUID)
    end
end

function hcTk.ArgusSoulburstApplied(guid)
    if (hcTk.Group[guid] ~= nil) then
        hcTk.Encounter["SoulburstTargets"][#hcTk.Encounter["SoulburstTargets"] + 1] = guid
        hcTk.Encounter["SoulExpire"] = GetTime() + 15

        if (hcTk.GenericGetBossOption("Antorus", "Argus", "SoulBurstAndBombDisplay")) then
            HCToolkitGroupFrame:SetScript("OnUpdate", hcTk.ArgusPrintSouls)
        end

        if (hcTk.ArgusIsSoulMarker()) then
            SetRaidTarget(hcTk.Group[guid]["unit"], #hcTk.Encounter["SoulburstTargets"])  
        end
    end
end

function hcTk.ArgusSoulbombApplied(guid)
    if (hcTk.Group[guid] ~= nil) then
        hcTk.Encounter["SoulbombTarget"] = guid
        hcTk.Encounter["SoulExpire"] = GetTime() + 15

        if (hcTk.GenericGetBossOption("Antorus", "Argus", "SoulBurstAndBombDisplay")) then
            HCToolkitGroupFrame:SetScript("OnUpdate", hcTk.ArgusPrintSouls)
        end

        if (hcTk.ArgusIsSoulMarker()) then
           SetRaidTarget(hcTk.Group[guid]["unit"], 3) 
        end
    end
end

function hcTk.ArgusRemoveSouls(guid)
    if (hcTk.Encounter["SoulbombTarget"] == guid) then
        hcTk.Encounter["SoulbombTarget"] = nil
        if (hcTk.ArgusIsSoulMarker()) then
            SetRaidTarget(hcTk.Group[guid]["unit"], 0)
        end
    else 
        for k, g in ipairs(hcTk.Encounter["SoulburstTargets"]) do
            if (guid == g) then
                table.remove(hcTk.Encounter["SoulburstTargets"], k)
                if (hcTk.ArgusIsSoulMarker()) then
                    SetRaidTarget(hcTk.Group[guid]["unit"], 0)
                end
            end
        end
    end
end

function hcTk.ArgusPrintSouls()
    local strs = {}

    for k, guid in ipairs(hcTk.Encounter["SoulburstTargets"]) do
        strs[#strs + 1] = hcTk.ArgusCreateSoulString(guid, k)
    end

    if (hcTk.Encounter["SoulbombTarget"] ~= nil) then
        strs[#strs + 1] = hcTk.ArgusCreateSoulString(hcTk.Encounter["SoulbombTarget"], 3)
    end

    hcTk.FrameUpdateText(HCToolkitGroupFrame, table.concat(strs, "\n"))

    if (hcTk.Encounter["SoulExpire"] == nil or GetTime() > hcTk.Encounter["SoulExpire"]) then
        hcTk.Encounter["SoulExpire"] = nil
        HCToolkitGroupFrame:SetScript("OnUpdate", nil)
        hcTk.FrameUpdateText(HCToolkitGroupFrame, "")
    end
end

function hcTk.ArgusCreateSoulString(guid, markerId)
    return string.format("%s %s", hcTk.MarkerIdToString(markerId), hcTk.UnitPlayerString(hcTk.Group[guid]))
end

function hcTk.ArgusIsSoulMarker()
    return UnitIsGroupLeader(hcTk.Group[hcTk.PlayerGUID]["unit"]) and hcTk.GenericGetBossOption("Antorus", "Argus", "MarkSoulBurstAndBomb")
end

hcTk.ArgusCombatLogFns = {
    ["SPELL_AURA_APPLIED"] = hcTk.ArgusSpellAuraApplied,
    ["SPELL_AURA_REMOVED"] = hcTk.ArgusSpellAuraRemoved,
    ["UNIT_DIED"] = hcTk.ArgusUnitDied
}